/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.frontend.data.set.taglib.internal.jaxrs.context.provider;

import com.liferay.frontend.data.set.provider.search.FDSPagination;
import com.liferay.portal.kernel.util.ParamUtil;

import javax.servlet.http.HttpServletRequest;

import javax.ws.rs.ext.Provider;

import org.apache.cxf.jaxrs.ext.ContextProvider;
import org.apache.cxf.message.Message;

import org.osgi.service.component.annotations.Component;

/**
 * @author Marco Leo
 */
@Component(immediate = true, service = PaginationContextProvider.class)
@Provider
public class PaginationContextProvider
	implements ContextProvider<FDSPagination> {

	@Override
	public FDSPagination createContext(Message message) {
		HttpServletRequest httpServletRequest =
			(HttpServletRequest)message.getContextualProperty("HTTP.REQUEST");

		int page = ParamUtil.getInteger(httpServletRequest, "page", 1);
		int pageSize = ParamUtil.getInteger(httpServletRequest, "pageSize", 20);

		return new FDSPaginationImpl(page, pageSize);
	}

	private class FDSPaginationImpl implements FDSPagination {

		public FDSPaginationImpl(int page, int pageSize) {
			_page = page;
			_pageSize = pageSize;
		}

		@Override
		public int getEndPosition() {
			return _page * _pageSize;
		}

		@Override
		public int getPage() {
			return _page;
		}

		@Override
		public int getPageSize() {
			return _pageSize;
		}

		@Override
		public int getStartPosition() {
			return (_page - 1) * _pageSize;
		}

		private final int _page;
		private final int _pageSize;

	}

}