/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.frontend.data.set.taglib.servlet.taglib;

import com.liferay.frontend.data.set.model.FDSPaginationEntry;
import com.liferay.frontend.data.set.taglib.internal.js.loader.modules.extender.npm.NPMResolverProvider;
import com.liferay.frontend.data.set.taglib.internal.servlet.ServletContextUtil;
import com.liferay.frontend.data.set.taglib.internal.util.ServicesProvider;
import com.liferay.frontend.js.loader.modules.extender.npm.NPMResolvedPackageNameUtil;
import com.liferay.frontend.js.loader.modules.extender.npm.NPMResolver;
import com.liferay.frontend.js.module.launcher.JSModuleResolver;
import com.liferay.portal.kernel.dao.search.SearchContainer;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.portlet.PortalPreferences;
import com.liferay.portal.kernel.portlet.PortletPreferencesFactoryUtil;
import com.liferay.portal.kernel.util.HashMapBuilder;
import com.liferay.portal.kernel.util.JavaConstants;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.template.react.renderer.ComponentDescriptor;
import com.liferay.portal.template.react.renderer.ReactRenderer;
import com.liferay.portal.util.PropsValues;
import com.liferay.taglib.util.AttributesTagSupport;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import javax.portlet.PortletResponse;
import javax.portlet.PortletURL;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;

/**
 * @author Marko Cikos
 */
public class BaseDisplayTag extends AttributesTagSupport {

	@Override
	public int doEndTag() throws JspException {
		try {
			return processEndTag();
		}
		catch (Exception exception) {
			throw new JspException(exception);
		}
		finally {
			doClearTag();
		}
	}

	@Override
	public int doStartTag() throws JspException {
		try {
			_fdsPaginationEntries = new ArrayList<>();

			for (int curDelta :
					PropsValues.SEARCH_CONTAINER_PAGE_DELTA_VALUES) {

				if (curDelta > SearchContainer.MAX_DELTA) {
					continue;
				}

				_fdsPaginationEntries.add(
					new FDSPaginationEntry(null, curDelta));
			}
		}
		catch (Exception exception) {
			_log.error(exception);
		}

		return super.doStartTag();
	}

	public Map<String, Object> getAdditionalProps() {
		return _additionalProps;
	}

	public String getId() {
		return _id;
	}

	public int getItemsPerPage() {
		return _itemsPerPage;
	}

	public String getNamespace() {
		if (_namespace != null) {
			return _namespace;
		}

		HttpServletRequest httpServletRequest = getRequest();

		PortletResponse portletResponse =
			(PortletResponse)httpServletRequest.getAttribute(
				JavaConstants.JAVAX_PORTLET_RESPONSE);

		if (portletResponse != null) {
			_namespace = portletResponse.getNamespace();
		}

		return _namespace;
	}

	public int getPageNumber() {
		return _pageNumber;
	}

	public String getPropsTransformer() {
		return _propsTransformer;
	}

	public String getRandomNamespace() {
		return _randomNamespace;
	}

	public List<Object> getSelectedItems() {
		return _selectedItems;
	}

	public void setAdditionalProps(Map<String, Object> additionalProps) {
		_additionalProps = additionalProps;
	}

	public void setId(String id) {
		_id = id;
	}

	public void setItemsPerPage(int itemsPerPage) {
		_itemsPerPage = itemsPerPage;
	}

	public void setNamespace(String namespace) {
		_namespace = namespace;
	}

	public void setPageNumber(int pageNumber) {
		_pageNumber = pageNumber;
	}

	/**
	 * @deprecated As of Cavanaugh (7.4.x), with no direct replacement
	 */
	@Deprecated
	public void setPortletURL(PortletURL portletURL) {
		_portletURL = portletURL;
	}

	public void setPropsTransformer(String propsTransformer) {
		_propsTransformer = propsTransformer;
	}

	public void setPropsTransformerServletContext(
		ServletContext propsTransformerServletContext) {

		_propsTransformerServletContext = propsTransformerServletContext;
	}

	public void setRandomNamespace(String randomNamespace) {
		_randomNamespace = randomNamespace;
	}

	public void setSelectedItems(List<Object> selectedItems) {
		_selectedItems = selectedItems;
	}

	protected void cleanUp() {
		_additionalProps = null;
		_fdsPaginationEntries = null;
		_id = null;
		_itemsPerPage = 0;
		_namespace = null;
		_pageNumber = 0;
		_portletURL = null;
		_propsTransformer = null;
		_propsTransformerServletContext = null;
		_randomNamespace = null;
		_selectedItems = null;
	}

	protected void doClearTag() {
		clearDynamicAttributes();
		clearParams();
		clearProperties();

		cleanUp();
	}

	protected ServletContext getPropsTransformerServletContext() {
		if (_propsTransformerServletContext != null) {
			return _propsTransformerServletContext;
		}

		return pageContext.getServletContext();
	}

	protected Map<String, Object> prepareProps(Map<String, Object> props) {
		return HashMapBuilder.<String, Object>putAll(
			props
		).put(
			"additionalProps",
			() -> {
				if (_additionalProps != null) {
					return _additionalProps;
				}

				return null;
			}
		).put(
			"customViews", _getCustomViews()
		).put(
			"namespace", getNamespace()
		).put(
			"pagination",
			HashMapBuilder.<String, Object>put(
				"deltas", _fdsPaginationEntries
			).put(
				"initialDelta", _itemsPerPage
			).put(
				"initialPageNumber", _pageNumber
			).build()
		).put(
			"selectedItems", _selectedItems
		).build();
	}

	protected int processEndTag() throws Exception {
		JspWriter jspWriter = pageContext.getOut();

		jspWriter.write("<div class=\"table-root\" id=\"");
		jspWriter.write(getRandomNamespace());
		jspWriter.write("table-id\"><span aria-hidden=\"true\" class=\"");
		jspWriter.write("loading-animation my-7\"></span>");

		NPMResolver npmResolver = NPMResolverProvider.getNPMResolver();

		String moduleName = npmResolver.resolveModuleName(
			"@liferay/frontend-data-set-web/FrontendDataSet");

		String propsTransformer = null;

		if (Validator.isNotNull(_propsTransformer)) {
			String resolvedPackageName = null;

			try {
				resolvedPackageName = NPMResolvedPackageNameUtil.get(
					getPropsTransformerServletContext());
			}
			catch (UnsupportedOperationException
						unsupportedOperationException) {

				if (_log.isDebugEnabled()) {
					_log.debug(unsupportedOperationException);
				}

				JSModuleResolver jsModuleResolver =
					ServicesProvider.getJSModuleResolver();

				resolvedPackageName = jsModuleResolver.resolveModule(
					getPropsTransformerServletContext(), null);
			}

			propsTransformer = resolvedPackageName + "/" + _propsTransformer;
		}

		ComponentDescriptor componentDescriptor = new ComponentDescriptor(
			moduleName, getId(), new LinkedHashSet<>(), false,
			propsTransformer);

		ReactRenderer reactRenderer = ServicesProvider.getReactRenderer();

		reactRenderer.renderReact(
			componentDescriptor, prepareProps(new HashMap<>()), getRequest(),
			jspWriter);

		jspWriter.write("</div>");

		return EVAL_PAGE;
	}

	protected void setAttributes(HttpServletRequest httpServletRequest) {
	}

	private String _getCustomViews() {
		HttpServletRequest httpServletRequest = getRequest();

		PortalPreferences portalPreferences =
			PortletPreferencesFactoryUtil.getPortalPreferences(
				httpServletRequest);

		return portalPreferences.getValue(
			ServletContextUtil.getFDSSettingsNamespace(httpServletRequest, _id),
			"customViews", "{}");
	}

	private static final Log _log = LogFactoryUtil.getLog(BaseDisplayTag.class);

	private Map<String, Object> _additionalProps;
	private List<FDSPaginationEntry> _fdsPaginationEntries;
	private String _id;
	private int _itemsPerPage;
	private String _namespace;
	private int _pageNumber;
	private PortletURL _portletURL;
	private String _propsTransformer;
	private ServletContext _propsTransformerServletContext;
	private String _randomNamespace;
	private List<Object> _selectedItems;

}